%% Operational modal analysis on Hardanger bridge using FDD/EFDD methods

% Estimate the modal parameters for the Hardanger Bridge using FDD and
% EFDD.
%
% This script uses four functions:
%       - SVD.m
%       - PSD_estimation.m
%       - damping_estimation.m
%       - frequency_estimation.m

%% Load validated data
data_file = dir('/volumes/NTNU/Hardanger data/Validated_data/filter/*.mat');
k = 4; % Low wind vel.
% k = 10; % Medium winf vel.
% k = 16; % High wind vel.
load(['/volumes/NTNU/Hardanger data/Validated_data/filter/' strrep(data_file(k).name,'.mat','')]);
h = msgbox([data_file(k).name],'Loaded data file');
acc = acc_filter_resample;

N = length(acc(1,:));
if isprime(N) == 0;
    acc = acc(:,:);
    elseif isprime(N) == 1;
    acc = acc(:,1:N-1);
    N = N-1;
end
N_dofs = length(acc(:,1));
fs = 4;
t = 0:(4/fs):N/fs;
%--------------------------------------------------------------------------
%% #################### Operational modal analysis ########################
%--------------------------------------------------------------------------
%% FDD
% Generate the PSD
N_Hamming = 14;
overlap = 0.60;
window = round(N/N_Hamming); 
noverlap = round(overlap*window);
nfft = 0.5*N;
N_freq = 0.5*nfft+1;

G = zeros(N_dofs,N_dofs,N_freq);
for i = 1:N_dofs;
    for k = 1:N_dofs;
            [G(i,k,:),f] = cpsd(acc(i,:),acc(k,:),window,noverlap,nfft,fs);
    end
end

% Singular value decomposition
[U,S,V] = SVD(G,N_dofs,N_freq);
%% EFDD
% Peak number form singular value plot (max 4 peaks in modal analysis)
peak = [];

% Estimate the PSD and corresponding correlation functions
N_peaks = length(peak);
[sigma, MODE_ESTIMATE] = MAC(0.8,peak,U,S,N_peaks,N_dofs,N_freq);
G_SDOF = sigma;
corr_SDOF = zeros(N_peaks,length(sigma));
for j2 = 1:N_peaks;
   corr(j2,:) = (ifft(G_SDOF(j2,:),'symmetric')); 
end

% Normalize correlation functions
corr_norm = zeros(length(peak),length(sigma));
for k = 1:length(peak);
    corr_norm(k,:) = (corr(k,:)-mean(corr(k,:)))/corr(k,1);
end
%% Damping and frequency estimation

% Correlation interval
j = []; l = []; m = []; q = [];

% Damping estimation
xi = damping_estimation(corr_norm,j,l,m,q,peak);

% Frequency estimation
[wn,fn] = frequency_estimation(corr_norm,peak,j,l,m,q,xi,fs);

% Store damping ratio vector and undamped natural frequency
MP = [xi;fn;wn];
%--------------------------------------------------------------------------
%% ############################PLOTS#######################################
%--------------------------------------------------------------------------
%% Plot of singular values
figure(1);
semilogy(squeeze(S(1,1,:))); grid on; hold on;
semilogy(squeeze(S(2,2,:)));
semilogy(squeeze(S(3,3,:)));   
semilogy(squeeze(S(4,4,:)));
semilogy(squeeze(S(5,5,:)));
semilogy(squeeze(S(6,6,:)));