%% EFDD part II (of III)

% This function estimates the damping ratios by linear regression on the
% logarithmic decrements.

% Input:  corr_norm - normalized correlation
%         j, l, m, q - correlation interval for peak 1, 2, 3 and 4
%         peak - peak vector
%         peak_SDOF(1, 2, 3, 4) - peak from normalized correlation function
%         loc_SDOF(1, 2, 3, 4) - location of peaks
% Output: xi = [xi1 xi2 xi3 xi4] - estimated damping ratios  

function [xi] = damping_estimation(corr_norm,j,l,m,q,peak)

for i = 1:length(peak);
    if i ==1;
[peak_SDOF1 , loc_SDOF1] = findpeaks(abs(corr_norm(1,:)));
    elseif i == 2;
[peak_SDOF2 , loc_SDOF2] = findpeaks(abs(corr_norm(2,:)));
    elseif i == 3;
[peak_SDOF3 , loc_SDOF3] = findpeaks(abs(corr_norm(3,:)));
    elseif i == 4
[peak_SDOF4 , loc_SDOF4] = findpeaks(abs(corr_norm(4,:)));
    end
end

for i = 1:length(peak);
    if i == 1;
delta1 = zeros(1,length(peak_SDOF1));
for k = 1:length(peak_SDOF1)
    delta1(k) = (2/k) * log(corr_norm(1,1)/abs(peak_SDOF1(k)));
end

Y_delta1 = delta1.*loc_SDOF1 + 2*log(abs(peak_SDOF1));
P1 = polyfit(loc_SDOF1(j(1):j(2)),Y_delta1(j(1):j(2)),1);
Y1 = P1(1)*loc_SDOF1 + P1(2);
xi1 = P1(1)/sqrt(P1(1)^2+(2*pi)^2);

    elseif i == 2;
delta2 = zeros(1,length(peak_SDOF2));
for k = 1:length(peak_SDOF2)
    delta2(k) = (2/k) * log(corr_norm(2,1)/abs(peak_SDOF2(k)));
end

Y_delta2 = delta2.*loc_SDOF2 + 2*log(abs(peak_SDOF2));
P2 = polyfit(loc_SDOF2(l(1):l(2)),Y_delta2(l(1):l(2)),1);
Y2 = P2(1)*loc_SDOF2 + P2(2);
xi2 = P2(1)/sqrt(P2(1)^2+(2*pi)^2);

    elseif i == 3;
delta3 = zeros(1,length(peak_SDOF3));
for k = 1:length(peak_SDOF3)
    delta3(k) = (2/k) * log(corr_norm(3,1)/abs(peak_SDOF3(k)));
end
Y_delta3 = delta3.*loc_SDOF3 + 2*log(abs(peak_SDOF3));
P3 = polyfit(loc_SDOF3(m(1):m(2)),Y_delta3(m(1):m(2)),1);
Y3 = P3(1)*loc_SDOF3 + P3(2);
xi3 = P3(1)/sqrt(P3(1)^2+(2*pi)^2);

    elseif i == 4;
delta4 = zeros(1,length(peak_SDOF4));
for k = 1:length(peak_SDOF4)
    delta4(k) = (2/k) * log(corr_norm(4,1)/abs(peak_SDOF4(k)));
end
Y_delta4 = delta4.*loc_SDOF4 + 2*log(abs(peak_SDOF4));
P4 = polyfit(loc_SDOF4(q(1):q(2)),Y_delta4(q(1):q(2)),1);
Y4 = P4(1)*loc_SDOF4 + P4(2);
xi4 = P4(1)/sqrt(P4(1)^2+(2*pi)^2);

    end
end
% Estimated damping ratios
xi = [xi1 xi2 xi3 xi4];
end